/*
 *  Ascend Instance Tree Type Visit Implementation
 *  by Tom Epperly
 *  9/3/89
 *  Version: $Revision: 1.17 $
 *  Version control file: $RCSfile: visitinst.c,v $
 *  Date last modified: $Date: 97/09/09 22:33:44 $
 *  Last modified by: $Author: ballan $
 *
 *  This file is part of the Ascend Language Interpreter.
 *
 *  Copyright (C) 1996 Benjamin Allan
 *  based on instance.c
 *  Copyright (C) 1990, 1993, 1994 Thomas Guthrie Epperly
 *
 *  The Ascend Language Interpreter is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The Ascend Language Interpreter is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.
 *
 */

#include <stdarg.h>
#include "utilities/ascConfig.h"
#include "utilities/ascPanic.h"
#include "utilities/ascMalloc.h"
#include "general/list.h"
#include "general/dstring.h"
#include "compiler/compiler.h"
#include "compiler/symtab.h"
#include "compiler/fractions.h"
#include "compiler/dimen.h"
#include "compiler/functype.h"
#include "compiler/types.h"
#include "compiler/child.h"
#include "compiler/type_desc.h"
#include "compiler/instance_enum.h"
#include "compiler/instance_name.h"
#include "compiler/instquery.h"
#include "compiler/instance_io.h"
#include "compiler/instmacro.h"
#include "compiler/parentchild.h"
#include "compiler/when_util.h"
#include "compiler/instance_types.h"
#include "compiler/visitinst.h"

#ifndef lint
static CONST char InstanceVisitModuleID[] = "$Id: visitinst.c,v 1.17 97/09/09 22:33:44 ballan Exp $";
#endif

unsigned long global_visit_num = 0;
int g_iscomplete = 1;
/************* VisitInstance stuff **************************************/

#define OLDCHECKVISIT 0
#if OLDCHECKVISIT
/* returns 1 if not visited yet, 0 if been here already. */
static int CheckVisitNumber(struct Instance *i)
{
  AssertMemory(i);
  switch(i->t) {
  case MODEL_INST:
    if (global_visit_num > MOD_INST(i)->visited) {
      MOD_INST(i)->visited = global_visit_num;
      return 1;
    }
    else return 0;
  case REAL_CONSTANT_INST:
    if (global_visit_num > RC_INST(i)->visited) {
      RC_INST(i)->visited = global_visit_num;
      return 1;
    }
    else return 0;
  case BOOLEAN_CONSTANT_INST:
    if (global_visit_num > BC_INST(i)->visited) {
      BC_INST(i)->visited = global_visit_num;
      return 1;
    }
    else return 0;
  case INTEGER_CONSTANT_INST:
    if (global_visit_num > IC_INST(i)->visited) {
      IC_INST(i)->visited = global_visit_num;
      return 1;
    }
    else return 0;
  case SYMBOL_CONSTANT_INST:
    if (global_visit_num > SYMC_INST(i)->visited) {
      SYMC_INST(i)->visited = global_visit_num;
      return 1;
    }
    else return 0;
  case REAL_ATOM_INST:
    if (global_visit_num > RA_INST(i)->visited) {
      RA_INST(i)->visited = global_visit_num;
      return 1;
    }
    else return 0;
  case BOOLEAN_ATOM_INST:
    if (global_visit_num > BA_INST(i)->visited) {
      BA_INST(i)->visited = global_visit_num;
      return 1;
    }
    else return 0;
  case INTEGER_ATOM_INST:
    if (global_visit_num > IA_INST(i)->visited) {
      IA_INST(i)->visited = global_visit_num;
      return 1;
    }
    else return 0;
  case SET_ATOM_INST:
    if (global_visit_num > SA_INST(i)->visited) {
      SA_INST(i)->visited = global_visit_num;
      return 1;
    } else {
      return 0;
    }
  case SYMBOL_ATOM_INST:
    if (global_visit_num > SYMA_INST(i)->visited) {
      SYMA_INST(i)->visited = global_visit_num;
      return 1;
    } else {
      return 0;
    }
  case REL_INST:
    if (global_visit_num > RELN_INST(i)->visited) {
      RELN_INST(i)->visited = global_visit_num;
      return 1;
    } else {
      return 0;
    }
  case LREL_INST:
    if (global_visit_num > LRELN_INST(i)->visited) {
      LRELN_INST(i)->visited = global_visit_num;
      return 1;
    } else {
      return 0;
    }
  case WHEN_INST:
    if (global_visit_num > W_INST(i)->visited) {
      W_INST(i)->visited = global_visit_num;
      return 1;
    } else {
      return 0;
    }
  case ARRAY_INT_INST:
  case ARRAY_ENUM_INST:
    if (global_visit_num > ARY_INST(i)->visited) {
      ARY_INST(i)->visited = global_visit_num;
      return 1;
    } else {
      return 0;
    }
  case DUMMY_INST:
    if (global_visit_num > D_INST(i)->visited) {
      D_INST(i)->visited = global_visit_num;
      return 1;
    } else {
      return 0;
    }
  case REAL_INST:
  case INTEGER_INST:
  case BOOLEAN_INST:
  case SET_INST:
  case SYMBOL_INST:
    return 1;			/* assumed always unvisited */
        			/* since they have exactly one ATOM parent */
  default:
    Asc_Panic(2, "VisitInstanceTree",
              "Instance tree contains illegal instance.");
    break;
  }
}
#else  /* oldcheckvisit alternative, faster? needs testing */

/* returns 1 if not visited yet, 0 if been here already or i bogus. */
static int CheckVisitNumber(struct Instance *i)
{
  AssertMemory(i);
  if (i->t & IERRINST) {
    FPRINTF(ASCERR,"CheckVisitNumber called with bad instance\n");
    return 0;
  }
  /* models arrays atoms relations constants */
  if (IsCompoundInstance(i)) {
    /* models arrays, maybe lists in future */
    if (IsArrayInstance(i)) {
      /* arrays */
      if (global_visit_num > ARY_INST(i)->visited) {
        ARY_INST(i)->visited = global_visit_num;
        return 1;
      } else {
        return 0;
      }
    } else {
      /* models */
      if (i->t == MODEL_INST && global_visit_num > MOD_INST(i)->visited) {
        MOD_INST(i)->visited = global_visit_num;
        return 1;
      } else {
        return 0;
        /* sim instances are assumed visited, especially since they
         * should never be seen by VisitTree.
         */
      }
    }
  } else { /* atoms and relations constants */
    if (IsAtomicInstance(i)) {
      if (global_visit_num > CA_INST(i)->visited) {
        CA_INST(i)->visited = global_visit_num;
        return 1;
      } else {
        return 0;
      }
    }
    if (IsConstantInstance(i)) {
      /* constants */
      if (global_visit_num > CI_INST(i)->visited) {
        CI_INST(i)->visited = global_visit_num;
        return 1;
      } else {
        return 0;
      }
    }
    /* would be nice to have a CommonEqnInstance
     * here for whenrellogrel CE_INST.
     */
    if (i->t == REL_INST) {
      if (global_visit_num > RELN_INST(i)->visited) {
        RELN_INST(i)->visited = global_visit_num;
        return 1;
      } else {
        return 0;
      }
    }
    if (i->t == LREL_INST) {
      if (global_visit_num > LRELN_INST(i)->visited) {
        LRELN_INST(i)->visited = global_visit_num;
        return 1;
      } else {
        return 0;
      }
    }
    if (i->t == WHEN_INST) {
      if (global_visit_num > W_INST(i)->visited) {
        W_INST(i)->visited = global_visit_num;
        return 1;
      } else {
        return 0;
      }
    }
  }
  /* fundamentals and rogues */
  if (IsFundamentalInstance(i) ) {
    /* fundamentals assumed always unvisited */
    return 1;
  }
  if ( InstanceKind(i)==DUMMY_INST) {
    if (global_visit_num > D_INST(i)->visited) {
      D_INST(i)->visited = global_visit_num;
      return 1;
    } else {
      return 0;
    }
  } else {
    /* rogues */
    Asc_Panic(2, "VisitInstanceTree",
              "VisitInstanceTree: Instance tree contains illegal instance.");
    exit(2);/* Needed to keep gcc from whining */
  }
}
#endif  /*  OLDCHECKVISIT  */

#ifdef THIS_IS_AN_UNUSED_FUNCTION
/* if visit number of i is nonzero sets it 0 and returns 1, else 0 */
static int ZeroVisitNumber(struct Instance *i)
{
  AssertMemory(i);
  switch(i->t) {
  case MODEL_INST:
    if (MOD_INST(i)->visited) {
      MOD_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case REAL_CONSTANT_INST:
    if (RC_INST(i)->visited) {
      RC_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case BOOLEAN_CONSTANT_INST:
    if (BC_INST(i)->visited) {
      BC_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case INTEGER_CONSTANT_INST:
    if (IC_INST(i)->visited) {
      IC_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case SYMBOL_CONSTANT_INST:
    if (SYMC_INST(i)->visited) {
      SYMC_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case REAL_ATOM_INST:
    if (RA_INST(i)->visited) {
      RA_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case BOOLEAN_ATOM_INST:
    if (BA_INST(i)->visited) {
      BA_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case INTEGER_ATOM_INST:
    if (IA_INST(i)->visited) {
      IA_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case SET_ATOM_INST:
    if (SA_INST(i)->visited) {
      SA_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case SYMBOL_ATOM_INST:
    if (SYMA_INST(i)->visited) {
      SYMA_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case REL_INST:
    if (RELN_INST(i)->visited) {
      RELN_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case LREL_INST:
    if (LRELN_INST(i)->visited) {
      LRELN_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case WHEN_INST:
    if (W_INST(i)->visited) {
      W_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case ARRAY_INT_INST:
  case ARRAY_ENUM_INST:
    if (ARY_INST(i)->visited) {
      ARY_INST(i)->visited=0;
      return 1;
    }
    else return 0;
  case REAL_INST:
  case INTEGER_INST:
  case BOOLEAN_INST:
  case SET_INST:
  case SYMBOL_INST:
  case DUMMY_INST:
    return 0;			/* assumed always unvisited */
        			/* since they have exactly one ATOM parent */
  default:
    Asc_Panic(2, NULL, "ZeroVisitInstanceTree:"
              "  Instance tree contains illegal instance.\n");
    /*NOTREACHED*/
  }
}
#endif  /* THIS_IS_AN_UNUSED_FUNCTION */


static
void WriteWhereNull(FILE *f, struct Instance *i)
{
  (void) i;
  FPRINTF(f,"Null instance in tree at ????\n");
  g_iscomplete=0;
}

/* set global_visit_num = 0 after calling this for all instances
in simulation universe and prototype libraries. */
void ResetVisitCounts(struct Instance *inst)
{
  unsigned long nc,c;
  struct Instance *child;
  AssertMemory(inst);
  if (InstanceKind(inst) == SIM_INST) return; /* ack!! */
  if (CheckVisitNumber(inst)){
    /* subtree has been visited in the past, not 0d */
    if (NotAtom(inst)) {
      nc = NumberChildren(inst);
      for(c=1;c<=nc;c++) {
        if ((child = InstanceChild(inst,c))!=NULL) {
          ResetVisitCounts(child);
          /* don't really care about NULL */
        }
      }
    }
  }
}

static void SlowVisitTree(struct Instance *inst,
        		VisitProc proc,
        		int depth, int leaf)
{
  unsigned long nc,c;
  struct Instance *child;
  unsigned nullchildren=0;

  AssertMemory(inst);			/* If null got here, die */
  if (CheckVisitNumber(inst)){		/* Not here another way already.*/
    if (!depth) (*proc)(inst);	       /* Apply on the way down, if top down.*/
    if (leaf||NotAtom(inst)) {		/* Go down on all children not null.*/
      nc = NumberChildren(inst);
      for(c=1;c<=nc;c++) {
        if ((child = InstanceChild(inst,c))!=NULL) { /* don't go down NULL */
          SlowVisitTree(child,proc,depth,leaf);
        } else nullchildren++;
        /* no point in whining about someplace we can't give a name */
      }
      if (nullchildren) {
        FPRINTF(ASCERR,"Found %u NULL children of ",nullchildren);
        WriteInstanceName(ASCERR,inst,NULL);
        FPRINTF(ASCERR,".\n");
      }
    }
    if (depth) (*proc)(inst);		/* Apply on way up, if bottom up.*/
  }
}

void SlowVisitInstanceTree(struct Instance *inst,
        			VisitProc proc,
        			int depth, int leaf)
{
  global_visit_num++;
  AssertMemory(inst);
  if (inst!=NULL) {
    SlowVisitTree(inst,proc,depth,leaf);
  } else {
    FPRINTF(ASCERR,"SlowVisitInstanceTree called with NULL.");
  }
}

static
void FastVisitTree(struct Instance *inst, VisitProc proc,int depth, int leaf)
{
  unsigned long nc,c;
  struct Instance *child;
  AssertMemory(inst);
  if (CheckVisitNumber(inst)){
    if (!depth) (*proc)(inst);
    if (leaf||NotAtom(inst)) {
      nc = NumberChildren(inst);
      for(c=1;c<=nc;c++)
        if ((child = InstanceChild(inst,c))!=NULL) {
          FastVisitTree(child,proc,depth,leaf);
        } else {
          WriteWhereNull(ASCERR,inst);
        }
    }
    if (depth) (*proc)(inst);
  }
}

static
void SilentVisitTree(struct Instance *inst,VisitProc proc, int depth, int leaf)
{
  unsigned long nc,c;
  struct Instance *child;
  AssertMemory(inst);
  if (CheckVisitNumber(inst)){
    if (!depth) (*proc)(inst);
    if (leaf||NotAtom(inst)) {
      nc = NumberChildren(inst);
      for(c=1;c<=nc;c++) {
        if ((child = InstanceChild(inst,c))!=NULL) {
          SilentVisitTree(child,proc,depth,leaf);
        }
      }
    }
    if (depth) (*proc)(inst);
  }
}

void FastVisitInstanceTree(struct Instance *inst, VisitProc proc,
        	       int depth, int leaf)
{
  global_visit_num++;
  AssertMemory(inst);
  FastVisitTree(inst,proc,depth,leaf);
}

void SilentVisitInstanceTree(struct Instance *inst, VisitProc proc,
        	       int depth, int leaf)
{
  global_visit_num++;
  AssertMemory(inst);
  SilentVisitTree(inst,proc,depth,leaf);
}

#define IVIT_MIN_LEN 20
static
void IndexedVisitCheckSize(unsigned long **llist, unsigned int *llen, int len)
{
  unsigned long *old;
  if ( *llen < (unsigned int)len || !(*llen) || llist == NULL) {
    old = *llist;
    if (*llist != NULL) {
      *llist = (unsigned long *)ascrealloc(*llist,
                         sizeof(unsigned long)*(*llen+IVIT_MIN_LEN));
    } else {
      *llist = (unsigned long *)ascmalloc(sizeof(unsigned long)*IVIT_MIN_LEN);
    }
    if (*llist == NULL) {
      *llist = old;
      Asc_Panic(2,"IndexedVisitInstanceTree","insufficient memory");
    }
  }
}

static
void IndexedVisitTree(struct Instance *inst, IndexedVisitProc proc,
                      int depth, int leaf,
                      unsigned long **llist, unsigned int *llen, int len,
                      VOIDPTR userdata)
{
  unsigned long nc,c;
  struct Instance *child;
  int oldlen;

  AssertMemory(inst);
  if (CheckVisitNumber(inst)){
    if (!depth) (*proc)(inst,*llist,len,userdata);
    if (leaf||NotAtom(inst)) {
      oldlen = len;
      len++;
      nc = NumberChildren(inst);
      IndexedVisitCheckSize(llist,llen,len);
      for (c = 1; c <= nc; c++) {
        child = InstanceChild(inst,c);
        if (child != NULL) {
          (*llist)[oldlen] = c;
          IndexedVisitTree(child,proc,depth,leaf,llist,llen,len,userdata);
        }
      }
      len--;
    }
    if (depth) (*proc)(inst,*llist,len,userdata);
  }
}

void IndexedVisitInstanceTree(struct Instance *inst, IndexedVisitProc proc,
        	              int depth, int leaf, unsigned long **llist,
                              unsigned int *llen, VOIDPTR userdata)
{
  global_visit_num++;
  AssertMemory(inst);

  assert(llist != NULL);
  assert(llen != NULL);
  IndexedVisitCheckSize(llist,llen,IVIT_MIN_LEN);

  IndexedVisitTree(inst,proc,depth,leaf,llist,llen,0,userdata);
}
#undef IVIT_MIN_LEN

static
void SilentVisitTreeTwo(struct Instance *inst,
        	        VisitTwoProc proc,
        	        int depth, int leaf,VOIDPTR userdata)
{
  unsigned long nc,c;
  struct Instance *child;
  AssertMemory(inst);
  if (CheckVisitNumber(inst)){
    if (!depth) {
      (*proc)(inst,userdata);
    }
    if (leaf||NotAtom(inst)) {
      nc = NumberChildren(inst);
      for(c=1;c<=nc;c++) {
        if ( (child = InstanceChild(inst,c)) !=NULL) {
          SilentVisitTreeTwo(child,proc,depth,leaf,userdata);
        }
      }
    }
    if (depth) {
      (*proc)(inst,userdata);
    }
  }
}

static
void VisitTreeTwo(struct Instance *inst,
        	  VisitTwoProc proc,
        	  int depth, int leaf,VOIDPTR userdata)
{
  unsigned long nc,c;
  struct Instance *child;
  AssertMemory(inst);
  if (CheckVisitNumber(inst)){
    if (!depth) {
      (*proc)(inst,userdata);
    }
    if (leaf||NotAtom(inst)) {
      nc = NumberChildren(inst);
      for(c=1;c<=nc;c++) {
        if ( (child = InstanceChild(inst,c)) !=NULL) {
          VisitTreeTwo(child,proc,depth,leaf,userdata);
        } else  {
          WriteWhereNull(ASCERR,inst);
        }
      }
    }
    if (depth) {
      (*proc)(inst,userdata);
    }
  }
}


void SilentVisitInstanceTreeTwo(struct Instance *inst,
        		        VisitTwoProc proc,
        		        int depth, int leaf,
        		        VOIDPTR userdata)
{
  global_visit_num++;
  AssertMemory(inst);
  SilentVisitTreeTwo(inst,proc,depth,leaf,userdata);
}

void VisitInstanceTreeTwo(struct Instance *inst,
        		  VisitTwoProc proc,
        		  int depth, int leaf,
        		  VOIDPTR userdata)
{
  global_visit_num++;
  AssertMemory(inst);
  VisitTreeTwo(inst,proc,depth,leaf,userdata);
}

static
void SilentVisitRootsTwo(struct Instance *inst, VisitTwoProc proc,
        	         int depth,VOIDPTR userdata)
{
  unsigned long nc,c;
  struct Instance *parent;
  AssertMemory(inst);
  if (CheckVisitNumber(inst)) {
    if (!depth) {
      (*proc)(inst,userdata);
    }
    nc = NumberParents(inst);
    for (c=1; c <= nc; c++) {
      parent = InstanceParent(inst,c);
      assert(parent != NULL);
      SilentVisitRootsTwo(parent,proc,depth,userdata);
    }
    if (depth) {
      (*proc)(inst,userdata);
    }
  }
}

void SilentVisitInstanceRootsTwo(struct Instance *inst, VisitTwoProc proc,
        	                int depth, VOIDPTR userdata)
{
  global_visit_num++;
  AssertMemory(inst);
  SilentVisitRootsTwo(inst,proc,depth,userdata);
}
