/*
 *  BrowserMethod.c
 *  by Kirk Abbott and Ben Allan
 *  Created: 1/94
 *  Version: $Revision: 1.16 $
 *  Version control file: $RCSfile: BrowserMethod.c,v $
 *  Date last modified: $Date: 97/09/04 18:26:39 $
 *  Last modified by: $Author: ballan $
 *
 *  This file is part of the ASCEND Tcl/Tk interface
 *
 *  Copyright 1997, Carnegie Mellon University
 *
 *  The ASCEND Tcl/Tk interface is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The ASCEND Tcl/Tk interface is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.  COPYING is found in ../compiler.
 */


#include "tcl.h"
#include "utilities/ascConfig.h"
#include "utilities/ascMalloc.h"
#include "general/list.h"
#include "compiler/compiler.h"
#include "compiler/symtab.h"
#include "compiler/fractions.h"
#include "compiler/dimen.h"
#include "compiler/types.h"
#include "compiler/stattypes.h"
#include "compiler/slist.h"
#include "compiler/statio.h"
#include "compiler/proc.h"
#include "compiler/procio.h"
#include "compiler/name.h"
#include "compiler/instance_enum.h"
#include "compiler/instquery.h"
#include "compiler/atomvalue.h"
#include "compiler/initialize.h"
#include "compiler/child.h"
#include "compiler/type_desc.h"
#include "compiler/units.h"
#include "solver/slv_types.h"
#include "interface/HelpProc.h"
#include "interface/BrowserProc.h"
#include "interface/BrowserMethod.h"
#include "interface/UnitsProc.h"
#include "interface/Qlfdid.h"

#ifndef lint
static CONST char BrowserMethodID[] = "$Id: BrowserMethod.c,v 1.16 97/09/04 18:26:39 ballan Exp $";
#endif


#define MAXID 256

#ifndef MAXIMUM_STRING_LENGTH
#define MAXIMUM_STRING_LENGTH 2048
#endif
#define BRSTRINGMALLOC \
(char *)ascmalloc(MAXIMUM_STRING_LENGTH * sizeof(char))


static
int BrowWriteProcedureList(Tcl_Interp *interp, struct gl_list_t *pl)
{
  struct InitProcedure *ip;
  unsigned long len,c;
  char *tmps1;

  if(pl!=NULL) {
    tmps1 = (char *)ascmalloc((MAXIMUM_NUMERIC_LENGTH+1)*sizeof(char));
    assert(tmps1!=NULL);
    len = gl_length(pl);
    for(c=1;c<=len;c++) {
      ip = (struct InitProcedure *)gl_fetch(pl,c);
      strcpy(tmps1,ProcName(ip));
      if (c==1) {
        Tcl_AppendResult(interp,tmps1,(char *)NULL);
      } else {
        Tcl_AppendResult(interp," ",tmps1,(char *)NULL);
      }
    }
    ascfree(tmps1);
    return 0;
  } else {
    return 1;
  }
}

/*
 * Takes the form \"bproc [qlfdid]\"
 */
int Asc_BrowProcedureListCmd(ClientData cdata, Tcl_Interp *interp,
                         int argc, char *argv[])
{
  struct TypeDescription *desc;
  struct Instance *i;
  int result,status;

  (void)cdata;    /* stop gcc whine about unused parameter */

 if (( argc < 1 ) || ( argc > 2 )) {
    Tcl_SetResult(interp, "usage: bproc [qlfdid]", TCL_STATIC);
    return TCL_ERROR;
  }

  if ( argc == 2 ) {
    status = Asc_QlfdidSearch3(argv[1],0);
    if (status==0) { 		/* catch inst ptr */
      i = g_search_inst;
    } else { 			/* failed. bail out. */
      Tcl_SetResult(interp,
                    "Asc_BrowProcedureListCmd: Could not find instance.",
                    TCL_STATIC);
      return TCL_ERROR;
    }
  } else {
    i = g_curinst;
  }
  if(!i) {
    Tcl_SetResult(interp, "1", TCL_STATIC);
    return TCL_OK;
  }
  if (InstanceKind(i)!=MODEL_INST) {
    Tcl_SetResult(interp, "1", TCL_STATIC);
    return TCL_OK;
  }
  desc = InstanceTypeDesc(i);
  if (desc) {
    result = BrowWriteProcedureList(interp,GetInitializationList(desc));
    if (result==0) {
      return TCL_OK;
    } else {
      Tcl_SetResult(interp, "1", TCL_STATIC);
      return TCL_OK;
    }
  } else {
    Tcl_SetResult(interp, "Null Instance TypeDescription ????", TCL_STATIC);
    return TCL_ERROR;
  }
}

/*
 * Takes the form runproc [qlfdid] name. If only 2 args given,
 * will use  the current instance. Otherwise will search for the
 * qlfdid given (the 2nd arg) and root the instance queries from
 * there.
 */
int Asc_BrowRunInitializeCmd(ClientData cdata, Tcl_Interp *interp,
                         int argc, char *argv[])
{
  int status;
  struct Name *name=NULL;
  struct Instance *i;

  (void)cdata;    /* stop gcc whine about unused parameter */

  if (( argc < 2 ) || ( argc > 3 )) {
    Tcl_SetResult(interp, "wrong # args: Usage runproc [qlfdid] name",
                  TCL_STATIC);
    return TCL_ERROR;
  }
  if ( argc == 2 ) {		/* e.g., runproc clear */
    i = g_curinst;
    name = CreateIdName(argv[1]);
  } else {				/* e.g., runproc a.b.c clear */
    status = Asc_QlfdidSearch3(argv[1],0);
    if (status==0) { 		/* catch inst ptr */
      i = g_search_inst;
      name = CreateIdName(argv[2]);
    } else { 				/* failed. bail out. */
      Tcl_AppendResult(interp,"runproc: Asc_BrowRunInitializeCmd: ",
                       "Could not find instance.",(char *)NULL);
      return TCL_ERROR;
    }
  }

  if (i==NULL) {
    Tcl_SetResult(interp, "Instance not found", TCL_STATIC);
    DestroyName(name);
    return TCL_ERROR;
  }
  if (Initialize(i,name)!=Proc_all_ok) {
    Tcl_SetResult(interp, "Error executing method", TCL_STATIC);
    DestroyName(name);
    return TCL_ERROR;
  } else {
    DestroyName(name);
    return TCL_OK;
  }
}

static void lowerstring(register char *str)
{
  while (*str != '\0') {
    if ((*str >= 'A')&&(*str <= 'Z')) {
      *str = *str + ('a' - 'A');
    }
    str++;
  }
}


static
int BrowDoAssignment(Tcl_Interp *interp,struct Instance *i,
                     char *value_str, char *unit_str)
{
  char buffer[MAXID], *tmps;
  int code = 0;
  switch(InstanceKind(i)) {
  case REAL_ATOM_INST:
  case REAL_INST:
  case REAL_CONSTANT_INST:
    code = Asc_UnitSetRealAtomValue(i,value_str,unit_str,0);
    switch (code) {
    case 0:
      break;
    case 1:
      Tcl_SetResult(interp, "Unparseable units given - Not assigned.",
                    TCL_STATIC);
      return TCL_ERROR;
    case 2:
      Tcl_SetResult(interp, "Dimensionally incompatible units - Not assigned.",
                    TCL_STATIC);
      return TCL_ERROR;
    case 3:
      Tcl_SetResult(interp,"Overflow in converting to SI value--Not assigned.",
                    TCL_STATIC);
      return TCL_ERROR;
    case 5:
      Tcl_SetResult(interp, "Unparseable value given - Not assigned.",
                    TCL_STATIC);
      return TCL_ERROR;
    default:
      return TCL_ERROR;
    }
    break;
  case BOOLEAN_ATOM_INST:
  case BOOLEAN_INST:
  case BOOLEAN_CONSTANT_INST:
    tmps = strcpy(buffer,value_str);
    lowerstring(tmps);
    if(strcmp(tmps,"true")==0 || strcmp(tmps,"1")==0 || strcmp(tmps,"yes")==0){
      SetBooleanAtomValue(i,1,0);
    } else if (strcmp(tmps,"false")==0
               || strcmp(tmps,"0")==0
               || strcmp(tmps,"no")==0) {
      SetBooleanAtomValue(i,0,0);
    } else {
      Tcl_SetResult(interp, "Incorrect boolean value", TCL_STATIC);
      return TCL_ERROR;
    }
    break;
  case INTEGER_ATOM_INST:
  case INTEGER_INST:
  case INTEGER_CONSTANT_INST:
    if (AtomMutable(i) || !AtomAssigned(i)) {
      SetIntegerAtomValue(i,atol(value_str),0);
    } else {
      Tcl_SetResult(interp, "Attempting to assign to an immutable integer",
                    TCL_STATIC);
      return TCL_ERROR;
    }
    break;
  case SYMBOL_ATOM_INST:
  case SYMBOL_INST:
  case SYMBOL_CONSTANT_INST:
    /* the symtab in symtab.c owns the string.
     * an instance only refers to it. */
    if (!AtomAssigned(i)) {
      tmps = (char *)AddSymbol(value_str); /* this will copy the string */
      SetSymbolAtomValue(i,tmps);
    }
    break;
  case SET_ATOM_INST:
  case SET_INST:
    /* not yet supported */
    break;
  default:
    Tcl_SetResult(interp, "The argument to assign is not a atom", TCL_STATIC);
    return TCL_ERROR;
  }
  return TCL_OK;
}

/*
 * This function should probably go away. !!.
 * We will at the next iteration.
 */
int Asc_BrowRunAssignQlfdidCmd2(ClientData cdata, Tcl_Interp *interp,
                            int argc, char *argv[])
{
  struct Instance *i;
  char *value_str = NULL;
  char *unit_str = NULL;
  int nok;

  (void)cdata;    /* stop gcc whine about unused parameter */

  if (( argc < 3 ) || ( argc > 4 )) {
    Tcl_AppendResult(interp,"wrong # args: ",
                     "Usage: \"qassgn2\" qlfdid value [units]",(char *)NULL);
    return TCL_ERROR;
  }
  nok = Asc_QlfdidSearch2(argv[1]);
  if (nok) { /* failed. bail out. */
    Tcl_AppendResult(interp," : Error -- Name not found",(char *)NULL);
    return TCL_ERROR;
  }
  i = g_search_inst;	/* catch inst ptr found in QlfdidSearch */
  value_str = argv[2];
  if ( argc == 4 ) {
    unit_str = argv[3];
  }
  if (strcmp("UNDEFINED",value_str)==0) {
    return TCL_OK;
  }
  nok = BrowDoAssignment(interp,i,value_str,unit_str);
  return nok;	/* whatever code returned by BrowDoAssignment */
}


int Asc_BrowRunAssignQlfdidCmd3(ClientData cdata, Tcl_Interp *interp,
                            int argc, char *argv[])
{
  struct Instance *i;
  char *value_str = NULL;
  char *unit_str = NULL;
  int nok;
  int relative = 0;

  (void)cdata;    /* stop gcc whine about unused parameter */

  if (( argc < 3 ) || ( argc > 5 )) {
    Tcl_AppendResult(interp,"wrong # args: ",
		     "Usage: \"qassgn3\" qlfdid value [units] [-relative]",
		     (char *)NULL);
    return TCL_ERROR;
  }
/* reading args out of order to get relative flag sorted out */
  if ( argc == 4 ) {
    if (strcmp("-relative",argv[3])==0) {
      relative = 1;
    } else {
      unit_str = argv[3];
    }
  }
  if ( argc == 5 ) {
    relative = 1;
  }

  nok = Asc_QlfdidSearch3(argv[1],relative);
  if (nok) { /* failed. bail out. */
    Tcl_AppendResult(interp," : Error -- Name not found",(char *)NULL);
    return TCL_ERROR;
  }
  i = g_search_inst;	/* catch inst ptr found in QlfdidSearch */
  value_str = argv[2];

  if (strcmp("UNDEFINED",value_str)==0) {
    return TCL_OK;
  }
  nok = BrowDoAssignment(interp,i,value_str,unit_str);
  return nok;	/* whatever code returned by BrowDoAssignment */
}


int Asc_BrowRunAssignmentCmd(ClientData cdata, Tcl_Interp *interp,
                         int argc, char *argv[])
{
  struct Instance *i;
  char *unit_str = NULL;
  char *value_str = NULL;
  int argstart=1;
  int nok;

  (void)cdata;    /* stop gcc whine about unused parameter */

  if (argc<2 || argc>4) {
    Tcl_AppendResult(interp, "Usage: \"", argv[0],
                     "\" [-search] value [units]",(char *)NULL);
    return TCL_ERROR;
  }
  if (argv[1][0] == '-') {
    if (strncmp("-search",argv[1],3)!=0) {
      Tcl_AppendResult(interp,"Error: ",argv[0]," Unknown option ",argv[1],
                       " want \"-search\"", (char *)NULL);
      return TCL_ERROR;
    } else {
      argstart++;
      i = g_search_inst;
    }
  } else {
    i = g_curinst;	/* use the current instance as the context */
  }
  if (!i) {
    Tcl_SetResult(interp, "Given instance is NULL", TCL_STATIC);
    return TCL_ERROR;
  }
  value_str = argv[argstart];
  if ( argc == 3 && argstart == 1) {
    unit_str = argv[2];
  }
  if ( argc == 4) {
    unit_str = argv[3];
  }
  if (strcmp("UNDEFINED",value_str)==0) {
    return TCL_OK;
  }
  nok = BrowDoAssignment(interp,i,value_str,unit_str);
  return nok;	/* whatever code returned by BrowDoAssignment */
}


int Asc_BrowWriteProcedure(ClientData cdata, Tcl_Interp *interp,
                       int argc, char *argv[])
{
  struct InitProcedure *proc;
  struct Instance *i;

  (void)cdata;    /* stop gcc whine about unused parameter */

  if ( argc < 3 || argc >4) {
    Tcl_SetResult(interp,"Usage bgetproc <methodname> <filepathname> [search]",
                  TCL_STATIC);
    return TCL_ERROR;
  }
  if (argc==4) {
    i = g_search_inst;
  } else {
    i = g_curinst;
  }
  if (i==NULL) {
    Tcl_SetResult(interp, "no instance sent to bgetproc", TCL_STATIC);
    return TCL_ERROR;
  }
  proc = FindProcedure(i,argv[1]);
  if (proc==NULL) {
    Tcl_SetResult(interp, "method named not found", TCL_STATIC);
    return TCL_ERROR;
  }
  if (argv[2][0]=='/') {
    FILE *fp=fopen(argv[2],"w");
    if (fp==NULL) {
       Tcl_SetResult(interp, "unable to open scratch file.", TCL_STATIC);
       return TCL_ERROR;
    }
    WriteProcedure(fp,proc);
    fclose(fp);
  } else {
    Tcl_SetResult(interp, "bgetproc given bad pathname.", TCL_STATIC);
    return TCL_ERROR;
  }
  return TCL_OK;
}


