/*
 *  LibraryProc.c
 *  by Kirk Abbott and Ben Allan
 *  Created: 1/94
 *  Version: $Revision: 1.29 $
 *  Version control file: $RCSfile: LibraryProc.c,v $
 *  Date last modified: $Date: 97/09/04 18:26:49 $
 *  Last modified by: $Author: ballan $
 *
 *  This file is part of the ASCEND Tcl/Tk interface
 *
 *  Copyright 1997, Carnegie Mellon University
 *
 *  The ASCEND Tcl/Tk interface is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The ASCEND Tcl/Tk interface is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.  COPYING is found in ../compiler.
 */

#include <time.h>
#include "tcl.h"
#include "utilities/ascConfig.h"
#include "utilities/ascMalloc.h"
#include "general/list.h"
#include "compiler/compiler.h"
#include "compiler/fractions.h"
#include "compiler/dimen.h"
#include "compiler/types.h"
#include "compiler/syntax.h"
#include "compiler/module.h"
#include "compiler/instance_enum.h"
#include "compiler/dump.h"
#include "compiler/stattypes.h"
#include "compiler/slist.h"
#include "compiler/child.h"
#include "compiler/type_desc.h"
#include "compiler/typedef.h"
#include "compiler/extfunc.h"
#include "compiler/library.h"
#include "compiler/prototype.h"
#include "solver/slv_types.h"
#include "interface/HelpProc.h"
#include "interface/LibraryProc.h"
#include "interface/Commands.h"
#include "interface/SimsProc.h"

#ifndef lint
static CONST char LibraryProcID[] = "$Id: LibraryProc.c,v 1.29 97/09/04 18:26:49 ballan Exp $";
#endif


extern
int Asc_FileIDCopy(FILE *filein, FILE *fileout)
{
  int c;
  while ((c = fgetc(filein)) != EOF) {
    fputc(c,fileout);
  }
  return 0;
}


STDHLF(Asc_LibrReadCmd,(Asc_LibrReadCmdHL,HLFSTOP));
extern int Asc_LibrReadCmd(ClientData cdata, Tcl_Interp *interp,
                           int argc, char **argv)
{
  struct module_t *mod;
  int relns_flag = 1;
  int result;
  int zz_parse();

  ASCUSE; /* see if first arg is -help */

  if ( argc < 2 || argc > 3 ) {
    Tcl_SetResult(interp,
                  "wrong # args: Usage: " Asc_LibrReadCmdHU, TCL_STATIC);
    return TCL_ERROR;
  }

  /* set up the parse relns flag */
  if ( argc == 3 ) {
    relns_flag = atoi(argv[2]);
  }

  SetParseRelnsFlag(relns_flag);
  if((mod = Asc_OpenModule(argv[1],NULL)) == NULL) {
    Tcl_AppendResult(interp,
                     Asc_LibrReadCmdHN ": Error in opening file ",
                     argv[1], (char*)NULL);
    result = TCL_ERROR;
  } else {
    /*
     * the open was successful.  parse the file.
     */
    Tcl_SetResult(interp, (char*)Asc_ModuleName(mod), TCL_VOLATILE);
    zz_parse();
    result = TCL_OK;
  }
  SetParseRelnsFlag(1);	  /* always reset */
  return result;
}


STDHLF(Asc_LibrModuleListCmd,(Asc_LibrModuleListCmdHL,HLFSTOP));
extern int Asc_LibrModuleListCmd(ClientData cdata, Tcl_Interp *interp,
                                 int argc, char **argv)
{
  unsigned long c;
  struct gl_list_t *ml;

  ASCUSE;  /* see if first arg is -help */

  ml = Asc_ModuleList();
  if( ml == NULL ) {
    /* module list is empty, return empty string */
    Tcl_ResetResult(interp);
    return TCL_OK;
  }

  for( c = gl_length(ml); c > 0; c-- ) {
    Tcl_AppendElement(interp, (char*)gl_fetch( ml, c ));
  }
  gl_destroy(ml);
  return TCL_OK;
}


STDHLF(Asc_LibrTypeListCmd,(Asc_LibrTypeListCmdHL,HLFSTOP));
extern int Asc_LibrTypeListCmd(ClientData cdata, Tcl_Interp *interp,
                               int argc, char **argv)
{
  struct gl_list_t *dl;
  unsigned long len;
  unsigned long c;
  CONST struct module_t *module;

  ASCUSE;  /* see if first arg is -help */

  if ( argc != 2 ) {
    Tcl_SetResult(interp, "wrong # args: Usage: " Asc_LibrTypeListCmdHU,
                  TCL_STATIC);
    return TCL_ERROR;
  }

  module = Asc_GetModuleByName(argv[1]);
  if( module == NULL ) {
    Tcl_AppendResult(interp, Asc_LibrTypeListCmdHN
                     ": Cannot find a module having the name ", argv[1],
                     NULL);
    return TCL_ERROR;
  }

  dl = Asc_TypeByModule(module);
  if ( dl == NULL ) {
    Tcl_AppendResult(interp, Asc_LibrTypeListCmdHN
                     ": The type definition list for", argv[1], "is NULL",
                     NULL);
    return TCL_ERROR;
  }

  len = gl_length(dl);
  for( c = 1; c <= len; c++ ) {
    Tcl_AppendElement(interp, (char*)gl_fetch(dl,c));
  }
  gl_destroy(dl);
  return TCL_OK;
}


STDHLF(Asc_LibrDestroyTypesCmd, (Asc_LibrDestroyTypesCmdHL,HLFSTOP));
extern int Asc_LibrDestroyTypesCmd(ClientData cdata, Tcl_Interp *interp,
                                   int argc, char **argv)
{
  ASCUSE;  /* see if first arg is -help */

  FFLUSH(stderr);
  DestroyLibrary();
  DestroyPrototype();
  EmptyTrash();
  Asc_DestroyModules();
  DefineFundamentalTypes();
  return TCL_OK;
}



/*
 *  void AddRootName(t);
 *      const struct TypeDescription *t;
 *
 *  AddRootName is called via gl_interate to find all root types, i.e,
 *  types that don't refine other types, and append the names of those
 *  types to the Tcl result.  We need to make a locally global pointer
 *  to the Tcl interpreter (called lroottypesinterp) so that AddRootName
 *  can access it.
 */
static Tcl_Interp *lroottypesinterp;
static void AddRootName(CONST struct TypeDescription *t)
{
  if(( t != NULL ) && ( GetRefinement(t) == NULL )) {
    Tcl_AppendElement(lroottypesinterp, (char *)GetName(t));
  }
}

STDHLF(Asc_LibrRootTypesCmd, (Asc_LibrRootTypesCmdHL, HLFSTOP));
extern int Asc_LibrRootTypesCmd(ClientData cdata, Tcl_Interp *interp,
                                int argc, char **argv)
{
  struct gl_list_t *deflist;

  ASCUSE;  /* see if first arg is -help */

  if ( argc != 1 ) {
    Tcl_SetResult(interp,
                  "wrong # args: Usage: " Asc_LibrRootTypesCmdHU, TCL_STATIC);
    return TCL_ERROR;
  }
  deflist = DefinitionList();
  lroottypesinterp = interp;
  gl_iterate(deflist,(void (*)(VOIDPTR))AddRootName);
  gl_destroy(deflist);
  return TCL_OK;
}


STDHLF(Asc_LibrCatalogCmd,(Asc_LibrCatalogCmdHL1,HLFSTOP));
extern int Asc_LibrCatalogCmd(ClientData cdata, Tcl_Interp *interp,
                              int argc, char **argv)
{
  struct gl_list_t *deflist;
  unsigned long len;
  unsigned long c;

  ASCUSE;  /* see if first arg is -help */

  if ( argc != 1 ) {
    Tcl_AppendResult(interp, Asc_LibrCatalogCmdHN, 
                     " takes no args", (char *)NULL);
    return TCL_ERROR;
  }
  deflist = DefinitionList();
  len = gl_length(deflist);
  for (c = 1; c <= len; c++) {
    Tcl_AppendElement(interp,
                      (char*)GetName((CONST struct TypeDescription*)
                                     gl_fetch(deflist,c)));
  }
  gl_destroy(deflist);
  return TCL_OK;
}



extern int Asc_GNUTextCmd(ClientData cdata, Tcl_Interp *interp,
                          int argc, char **argv)
{
  (void)cdata;    /* stop gcc whine about unused parameter */
  (void)argc;     /* stop gcc whine about unused parameter */
  (void)argv;     /* stop gcc whine about unused parameter */

  Tcl_AppendResult(interp,
                   "See the \"License\" buffer in the Script\n",
                   "for information on the GNU License and Warranty\n",
                   (char*)NULL);
  return TCL_OK;
  /*
   *  if ( argc == 2 ) {
   *    switch (argv[1][0]) {
   *    case 'l':
   *    case 'L':
   *      Tcl_AppendResult(interp, (char *)GPL1, (char *)GPL2, (char *)GPL3,
   *                       (char *)GPL4, (char *)GPL5, (char *)NULL);
   *      return TCL_OK;
   *    case 'w':
   *    case 'W':
   *    default :
   *      Tcl_AppendElement(interp,(char *)NO_WARRANTY);
   *      return TCL_OK;
   *    }
   *  } else {
   *    Tcl_AppendElement(interp,(char *)NO_WARRANTY);
   *    return TCL_OK;
   *  }
   */
}


STDHLF(Asc_LibrFindTypeCmd, (Asc_LibrFindTypeCmdHL,HLFSTOP));
extern int Asc_LibrFindTypeCmd(ClientData cdata, Tcl_Interp *interp,
                               int argc, char **argv)
{
  struct TypeDescription *desc;
  struct module_t *mod;

  ASCUSE;  /* see if first arg is -help */

  if ( argc != 2 ) {
    Tcl_SetResult(interp, "wrong # args: Usage: " Asc_LibrFindTypeCmdHU,
                  TCL_STATIC);
    return TCL_ERROR;
  }
  
  desc = FindType( argv[1] );
  if( desc == NULL ) {
    Tcl_AppendResult(interp, Asc_LibrFindTypeCmdHN
                     ": Type ", argv[1], " not found", NULL);
    return TCL_ERROR;
  }

  mod = GetModule(desc);
  if( mod == NULL ) {
    Tcl_AppendResult(interp, Asc_LibrFindTypeCmdHN
                     ": Type ", argv[1], " is a fundamental type", NULL);
    return TCL_ERROR;
  }

  Tcl_SetResult(interp, (char *)Asc_ModuleName(mod), TCL_VOLATILE);
  return TCL_OK;
}


STDHLF(Asc_LibrModuleInfoCmd,(Asc_LibrModuleInfoCmdHL,HLFSTOP));
extern int Asc_LibrModuleInfoCmd(ClientData cdata, Tcl_Interp *interp,
                                 int argc, char **argv)
{
  CONST struct module_t *mod;
  int i;

  ASCUSE;  /* see if first arg is -help */

  if( argc < 2 ) {
    Tcl_SetResult(interp, "wrong # args: Usage: " Asc_LibrModuleInfoCmdHU,
                  TCL_STATIC);
    return TCL_ERROR;
  }

  for( i = 1; i < argc; i++ ) {
    if((mod = Asc_GetModuleByName(argv[i])) != NULL ) {
      Tcl_AppendElement(interp, (char *)Asc_ModuleName(mod));
      Tcl_AppendElement(interp, (char *)Asc_ModuleFileName(mod));
      Tcl_AppendElement(interp, asctime(Asc_ModuleTimeModified(mod)));
    }
  }
  return TCL_OK;
}


STDHLF(Asc_LibrExternalFuncsCmd, (Asc_LibrExternalFuncsCmdHL,HLFSTOP));
extern int Asc_LibrExternalFuncsCmd(ClientData cdata, Tcl_Interp *interp,
                                    int argc, char **argv)
{
  char *stringresult;

  ASCUSE;  /* see if first arg is -help */

  if ( argc != 2 ) {
    Tcl_SetResult(interp, "Error: Usage libr_extfuncs <print,string>",
                  TCL_STATIC);
    return TCL_ERROR;
  }
  if (strncmp(argv[1],"print",1)==0) {
    FPRINTF(stdout,
            "-----------------External Functions Loaded------------------\n");
    PrintExtFuncLibrary(stdout);
    FPRINTF(stdout,
            "------------------------------------------------------------\n");
    return TCL_OK;
  }

  if (strncmp(argv[1],"string",1)==0) {
    stringresult = WriteExtFuncLibraryString();
    if (stringresult!=NULL) {
      Tcl_AppendResult(interp,stringresult,(char *)NULL);
      ascfree(stringresult);
    }
    return TCL_OK;
  }

  Tcl_AppendResult(interp,"invalid option to libr_extfuncs");
  return TCL_ERROR;
}



STDHLF(Asc_LibrHideTypeCmd, (Asc_LibrHideTypeCmdHL,HLFSTOP));
extern int Asc_LibrHideTypeCmd(ClientData cdata, Tcl_Interp *interp,
                               int argc, char **argv)
{
  struct TypeDescription *type;
  ChildListPtr clist;
  unsigned long c;

  ASCUSE;  /* see if first arg is -help */

  if( argc < 2 || argc > 3 ) {
    Tcl_SetResult(interp, "wrong # args: Usage: " Asc_LibrHideTypeCmdHU,
                  TCL_STATIC);
    return TCL_ERROR;
  }

  type = FindType(argv[1]);
  if (type==NULL) {
    Tcl_AppendResult(interp,
                     Asc_LibrHideTypeCmdHN " called with unknown type: ",
                     argv[1], (char *)NULL);
    return TCL_ERROR;
  }

  if ( argc == 2 ) {
    SetTypeShowBit(type,FALSE);
    return TCL_OK;
  }

  clist = GetChildList(type);
  if (clist==NULL) {
    Tcl_AppendResult(interp,
                     Asc_LibrHideTypeCmdHN " called with unknown type part",
                     (char *)NULL);
    return TCL_ERROR;
  }
  c = ChildPos(clist,argv[2]);
  if( c == 0UL ) {
    Tcl_AppendResult(interp,
                     Asc_LibrHideTypeCmdHN " called with unknown type part",
                     (char *)NULL);
    return TCL_ERROR;
  }
  ChildHide(clist,c);
  return TCL_OK;
}


STDHLF(Asc_LibrUnHideTypeCmd, (Asc_LibrUnHideTypeCmdHL,HLFSTOP));
extern int Asc_LibrUnHideTypeCmd(ClientData cdata, Tcl_Interp *interp,
                                 int argc, char **argv)
{
  struct TypeDescription *type;
  ChildListPtr clist;
  unsigned long c;

  ASCUSE;  /* see if first arg is -help */

  if (argc < 2 || argc > 3) {
    Tcl_SetResult(interp, "wrong # args: Usage: " Asc_LibrUnHideTypeCmdHU,
                  TCL_STATIC);
    return TCL_ERROR;
  }

  type = FindType(argv[1]);
  if (type==NULL) {
    Tcl_AppendResult(interp,
                     Asc_LibrUnHideTypeCmdHN " called with unknown type: ",
                     argv[1], (char*)NULL);
    return TCL_ERROR;
  }

  if ( argc == 2 ) {
    SetTypeShowBit(type,TRUE);
    return TCL_OK;
  }

  clist = GetChildList(type);
  if (clist==NULL) {
    Tcl_AppendResult(interp,
                     Asc_LibrUnHideTypeCmdHN " called with unknown type part",
                     (char*)NULL);
    return TCL_ERROR;
  }
  c = ChildPos(clist,argv[2]);
  if (c == 0UL) {
    Tcl_AppendResult(interp,
                     Asc_LibrUnHideTypeCmdHN " called with unknown type part",
                     (char*)NULL);
    return TCL_ERROR;
  }
  ChildShow(clist,c);
  return TCL_OK;
}



STDHLF(Asc_LibrGetFundamentalsCmd, (Asc_LibrGetFundamentalsCmdHL,HLFSTOP));
extern int Asc_LibrGetFundamentalsCmd(ClientData cdata, Tcl_Interp *interp,
                                      int argc, char **argv)
{
  struct gl_list_t *fundies;
  struct TypeDescription *type;
  unsigned long len,c,length;
  symchar *name;

  ASCUSE;  /* see if first arg is -help */

  if ( argc != 1 ) {
    Tcl_SetResult(interp,
                  "wrong # args: Usage: " Asc_LibrGetFundamentalsCmdHU,
                  TCL_STATIC);
    return TCL_ERROR;
  }

  fundies = FindFundamentalTypes();
  len = gl_length(fundies);

  for (c=1;c<=len;c++) {
    char *tmp;
    type = (struct TypeDescription *)gl_fetch(fundies,c);
    Tcl_AppendResult(interp,"{",(char *)NULL);
    name = GetName(type);
    length = (unsigned long)strlen(name);
    length++;
    tmp = (char *)ascmalloc(length *sizeof(char));
    strcpy(tmp,name);
    Tcl_AppendResult(interp,tmp,(char *)NULL);
    ascfree(tmp);
    Tcl_AppendResult(interp,"} ",(char *)NULL);
  }
  gl_destroy(fundies);
  return TCL_OK;
}


STDHLF(Asc_LibrTypeIsShownCmd, (Asc_LibrTypeIsShownCmdHL, HLFSTOP));
extern int Asc_LibrTypeIsShownCmd(ClientData cdata, Tcl_Interp *interp,
                                  int argc, char **argv)
{
  struct TypeDescription *type;
  char buf[MAXIMUM_NUMERIC_LENGTH];   /* string to hold integer */

  ASCUSE;  /* see if first arg is -help */

  if ( argc != 2 ) {
    Tcl_SetResult(interp,
                  "wrong # args: Usage: " Asc_LibrTypeIsShownCmdHU,
                  TCL_STATIC);
    return TCL_ERROR;
  }
  type = FindType( argv[1] );
  if( type == NULL ) {
    Tcl_AppendResult(interp,
                     Asc_LibrTypeIsShownCmdHN " called with unknown type",
                     argv[1], (char *)NULL);
    return TCL_ERROR;
  }
  sprintf(buf,"%d",TypeShow(type));
  Tcl_SetResult(interp, buf, TCL_VOLATILE);
  return TCL_OK;
}



STDHLF(Asc_LibrFileExtsCmd,(Asc_LibrFileExtsCmdHL,HLFSTOP));
int Asc_LibrFileExtsCmd(ClientData cdata, Tcl_Interp *interp,
                        int argc, char **argv)
{
  int i;

  ASCUSE;  /* see if first arg is -help */

  for (i = 0; i <  MOD_FILE_EXTS; i++) {
    Tcl_AppendElement(interp,(char *)g_alt_ending[i]);
  }
  return TCL_OK;
}

