/*
 *  LibraryProc.h
 *  by Kirk Abbott and Ben Allan
 *  Created: 1/94
 *  Version: $Revision: 1.13 $
 *  Version control file: $RCSfile: LibraryProc.h,v $
 *  Date last modified: $Date: 97/09/04 18:26:52 $
 *  Last modified by: $Author: ballan $
 *
 *  This file is part of the ASCEND Tcl/Tk interface
 *
 *  Copyright 1997, Carnegie Mellon University
 *
 *  The ASCEND Tcl/Tk interface is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The ASCEND Tcl/Tk interface is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.  COPYING is found in ../compiler.
 */

#ifndef LibraryProc_module_loaded
#define LibarryProc_module_loaded


/*
 *  To include this header, you must include the following:
 *      #include <stdio.h>
 *      #include "tcl.h"
 *      #include "interface/LibraryProc.h"
 */


extern int Asc_FileIDCopy(FILE *, FILE *);
/*
 *  int Asc_FileIDCopy
 *  FILE *filein;
 *  FILE *fileout;
 *  Copies the input from a file with specified file id to the other.
 *  It is the responsibility of the user to ensure that the files are
 *  open and writable.
 */


STDHLF_H(Asc_LibrReadCmd);
extern int Asc_LibrReadCmd(ClientData, Tcl_Interp*, int, char**);
/*  Registered as: */
#define Asc_LibrReadCmdHN "librread"
/*  Usage: */
#define Asc_LibrReadCmdHU \
  Asc_LibrReadCmdHN " <filename> [parse_relations]"
#define Asc_LibrReadCmdHS \
  "Opens and parses filename, a file containing ASCEND type definitions"
#define Asc_LibrReadCmdHL "\
 * Opens and parses filename; relations will be parsed unless the\n\
 * optional argument parse_relations is zero.  If the open is successful,\n\
 * the name of the module is returned, otherwise an error is reported.\n\
 * it is impossible to detect if there were parse errors from the exit\n\
 * status of this command.\n\
"



STDHLF_H(Asc_LibrModuleListCmd);
extern int Asc_LibrModuleListCmd(ClientData, Tcl_Interp*, int, char**);
/*  Registered as:  */
#define Asc_LibrModuleListCmdHN "libr_modules"
/*  Usage:  */
#define Asc_LibrModuleListCmdHU \
  Asc_LibrModuleListCmdHN " takes no arguments"
#define Asc_LibrModuleListCmdHS \
  "Returns a list of modules that have types defined"
#define Asc_LibrModuleListCmdHL "\
  * Returns a list of modules that have types defined\n\
"



STDHLF_H(Asc_LibrTypeListCmd);
extern int Asc_LibrTypeListCmd(ClientData, Tcl_Interp*, int, char**);
/*  Registered as:  */
#define Asc_LibrTypeListCmdHN "libr_types_in_module"
/*  Usage:  */
#define Asc_LibrTypeListCmdHU \
  Asc_LibrTypeListCmdHN " <module>"
#define Asc_LibrTypeListCmdHS \
  "Returns a list of types defined in the module named \"module\""
#define Asc_LibrTypeListCmdHL "\
  * Returns a list of types defined in the module named \"module\"\n\
"



STDHLF_H(Asc_LibrModuleInfoCmd);
extern int Asc_LibrModuleInfoCmd(ClientData, Tcl_Interp*, int, char**);
/* Registered as: */
#define Asc_LibrModuleInfoCmdHN "libr_moduleinfo"
/*  Usage:  */
#define Asc_LibrModuleInfoCmdHU \
  Asc_LibrModuleInfoCmdHN " <module> [<module>...]"
#define Asc_LibrModuleInfoCmdHS \
  "For each module, returns the module name, filename, and time modified."
#define Asc_LibrModuleInfoCmdHL "\
 *  For each module, returns the module name, filename, and time modified.\n\
"



STDHLF_H(Asc_LibrFindTypeCmd);
extern int Asc_LibrFindTypeCmd(ClientData, Tcl_Interp*, int, char**);
/* Registered as: */
#define Asc_LibrFindTypeCmdHN "libr_find_type"
/*  Usage:  */
#define Asc_LibrFindTypeCmdHU \
  Asc_LibrFindTypeCmdHN " <type>"
#define Asc_LibrFindTypeCmdHS \
  "Returns the name of the module where \"type\" is defined."
#define Asc_LibrFindTypeCmdHL "\
 *  Returns the name of the module where \"type\" is defined.\n\
"



STDHLF_H(Asc_LibrDestroyTypesCmd);
extern int Asc_LibrDestroyTypesCmd(ClientData, Tcl_Interp*, int, char**);
/* Registered as: */
#define Asc_LibrDestroyTypesCmdHN "libr_destroy_types"
/*  Usage:  */
#define Asc_LibrDestroyTypesCmdHU \
  Asc_LibrDestroyTypesCmdHN " takes no arguments"
#define Asc_LibrDestroyTypesCmdHS \
  "Reinitializes all base types and the module list"
#define Asc_LibrDestroyTypesCmdHL "\
 *  calls DestroyLibrary from the compiler, redefines basetypes,\n\
 *  and reinitializes interface module list.\n\
 *  Empties prototype library.\n\
"



STDHLF_H(Asc_LibrRootTypesCmd);
extern int Asc_LibrRootTypesCmd(ClientData, Tcl_Interp*, int, char**);
/* Registered as: */
#define Asc_LibrRootTypesCmdHN "libr_roottypes"
/*  Usage:  */
#define Asc_LibrRootTypesCmdHU \
  Asc_LibrRootTypesCmdHN " takes no arguments"
#define Asc_LibrRootTypesCmdHS \
  "Returns a list of types that are not refinements of other types"
#define Asc_LibrRootTypesCmdHL "\
 *  Returns a list of all types (base or otherwise) that are not\n\
 *  refinements of other types. These may be the roots of hierarchies.\n\
 *  Asking whether a type is, in fact, refined is a bit more expensive.\n\
"


STDHLF_H(Asc_LibrCatalogCmd);
extern int Asc_LibrCatalogCmd(ClientData, Tcl_Interp*, int, char**);
/*
 * status = Asc_LibrCatalogCmd(cdata,interp,argc,argv);
 * This needs to be combined with most of the rest of the
 * library into a command with options.
 * Registered as: */
#define Asc_LibrCatalogCmdHN "libr_catalog"
/* Usage: */
#define Asc_LibrCatalogCmdHU \
  Asc_LibrCatalogCmdHN " takes no options."
#define Asc_LibrCatalogCmdHS \
 "Returns the list of all known MODEL, and atomic type names."
#define Asc_LibrCatalogCmdHL1 "\
 * Each name can be the subject of subsequent queries. The list\n\
 * does not include the unnamed implicit types from relations, arrays,\n\
 * and other sources.\n\
"

extern int Asc_GNUTextCmd(ClientData, Tcl_Interp*, int, char**);
/*
 *  Asc_GNUTextCmd
 *  Registered as : gnutext
 *  gnutext [l,w] returns the string requested from
 *  Tom's license.c. defaults to warranty
 */



STDHLF_H(Asc_LibrExternalFuncsCmd);
extern int Asc_LibrExternalFuncsCmd(ClientData, Tcl_Interp*, int, char**);
#define Asc_LibrExternalFuncsCmdHN "libr_extfuncs"
/* Usage: */
#define Asc_LibrExternalFuncsCmdHU \
  Asc_LibrExternalFuncsCmdHN " takes no options."
#define Asc_LibrExternalFuncsCmdHS \
 "Prints the external function library"
#define Asc_LibrExternalFuncsCmdHL "\
 *  print :  Will print the external function library to stdout\n\
 *  print is the only option now built.\n\
"



STDHLF_H(Asc_LibrHideTypeCmd);
extern int Asc_LibrHideTypeCmd(ClientData, Tcl_Interp*, int, char**);
/* Registered as: */
#define Asc_LibrHideTypeCmdHN "libr_hide_type"
/*  Usage:  */
#define Asc_LibrHideTypeCmdHU \
  Asc_LibrHideTypeCmdHN " <type> [part]"
#define Asc_LibrHideTypeCmdHS \
  "Causes the instances of type not to be shown when browsing"
#define Asc_LibrHideTypeCmdHL "\
 *  Will set to FALSE the bit TYPESHOW of the Type Description type.\n\
 *  This will cause that the instances of type will not be shown for\n\
 *  browsing purposes.\n\
 *  If part is given, the part named will be hidden when browsing objects\n\
 *  the type given. The part name must be simple, that is no [] or . are\n\
 *  allowed. Parts of arrays cannot be hidden by subscript.\n\
"



STDHLF_H(Asc_LibrUnHideTypeCmd);
extern int Asc_LibrUnHideTypeCmd(ClientData, Tcl_Interp*, int, char**);
/* Registered as: */
#define Asc_LibrUnHideTypeCmdHN "libr_unhide_type"
/*  Usage:  */
#define Asc_LibrUnHideTypeCmdHU \
  Asc_LibrUnHideTypeCmdHN " <type> [part]"
#define Asc_LibrUnHideTypeCmdHS \
  "Causes the instances of type to be shown when browsing"
#define Asc_LibrUnHideTypeCmdHL "\
 *  Will set to TRUE the bit TYPESHOW of the Type Description type.\n\
 *  This will cause that the instances of type will be shown for\n\
 *  browsing purposes.\n\
 *  If part is given, the part named will be shown when browsing objects\n\
 *  the type given.\n\
"



STDHLF_H(Asc_LibrGetFundamentalsCmd);
extern int Asc_LibrGetFundamentalsCmd(ClientData, Tcl_Interp*, int, char**);
/* Registered as: */
#define Asc_LibrGetFundamentalsCmdHN "libr_get_fundamentals"
/*  Usage:  */
#define Asc_LibrGetFundamentalsCmdHU \
  Asc_LibrGetFundamentalsCmdHN " takes no arguments"
#define Asc_LibrGetFundamentalsCmdHS \
  "Will return a list containing the names of the fundamental types"
#define Asc_LibrGetFundamentalsCmdHL "\
 *  Will return a list containing the names of the fundamental types\n\
"



STDHLF_H(Asc_LibrTypeIsShownCmd);
extern int Asc_LibrTypeIsShownCmd(ClientData, Tcl_Interp*, int, char**);
/* Registered as: */
#define Asc_LibrTypeIsShownCmdHN "libr_type_is_shown"
/*  Usage:  */
#define Asc_LibrTypeIsShownCmdHU \
  Asc_LibrTypeIsShownCmdHN " <type>"
#define Asc_LibrTypeIsShownCmdHS \
  "Returns 1 if type is being shown, or 0 if not."
#define Asc_LibrTypeIsShownCmdHL "\
 *  Returns 1 is the TYPESHOWN bit of a type description is ON. 0 if is\n\
 *  OFF. Query required for updating the Library Display Buttons.\n\
"



STDHLF_H(Asc_LibrFileExtsCmd);
extern int Asc_LibrFileExtsCmd(ClientData, Tcl_Interp*, int, char**);
/* Registered as: */
#define Asc_LibrFileExtsCmdHN "libr_file_exts"
/*  Usage:  */
#define Asc_LibrFileExtsCmdHU \
  "libr_file_exts [all arguments simply ignored]"
#define Asc_LibrFileExtsCmdHS \
  "returns a list of the expected extensions for ascend MODEL files"
#define Asc_LibrFileExtsCmdHL \
"\
 * Returns the expected extensions for parsable code.\n\
 * Extensions other than these are probably not ascend MODEL/ATOM/unit code.\n\
 * The compiler will dutifully attempt to eat any file of any name; it is\n\
 * not restricted to any particular extension, or lack of extension.\n\
"
/* */




#endif /*LibraryProc_module_loaded*/





