/*
 *  Utility functions for Ascend
 *  Version: $Revision: 1.1 $
 *  Version control file: $RCSfile: old_utils.h,v $
 *  Date last modified: $Date: 97/07/18 11:49:50 $
 *  Last modified by: $Author: mthomas $
 *  Part of Ascend
 *
 *  This file is part of the Ascend Programming System.
 *
 *  Copyright (C) 1990 Karl Michael Westerberg
 *  Copyright (C) 1993 Joseph James Zaher
 *  Copyright (C) 1993, 1994 Benjamin Andrew Allan, Joseph James Zaher
 *
 *  The Ascend Programming System is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  Ascend is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software Foundation,
 *  Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check the file named
 *  COPYING.
 *
 *  This module defines the dimensionality checking and some other
 *  auxillaries for Ascend.
 *
 *
 *  This file is called old_utils because these utilities are outdated and
 *                      ^^^
 *  need to go away.  DO NOT MAKE ANY MORE REFERENCES TO THESE FUNCTIONS!
 */

/*
 *  Contents:     ASCEND Utilities module
 *
 *  Authors:      Karl Westerberg
 *                Joseph Zaher
 *
 *  Dates:        06/90 - original version
 *                03/94 - Re-wrote name making functions which no longer
 *                        need pre-allocated string space as an argument
 *                        to write to.  Added an additional function which
 *                        creates instance names utilizing the shortest
 *                        path.  The dimension string output function also
 *                        no longer requires pre-allocation of a string.
 *                04/94 - Provided a relation dimension checker.
 *
 *  Description:  This module provides supplementary functions which may
 *                prove useful by any client of the ASCEND system.
 *
 */
#ifndef _OLD_UTILS_H
#define _OLD_UTILS_H

/* requires
 * #include "instance_enum.h"
 * #include "fractions.h"
 * #include "dimen.h"
*/

extern char *asc_make_any_instance_name();
extern char *asc_make_short_instance_name();
/*
 *  name = asc_make_any_instance_name(inst,ref)
 *  name = asc_make_short_instance_name(inst,ref)
 *  char *name;
 *  struct Instance *inst, *ref;
 *
 *  asc_make_short_instance_name():
 *  A string will be created with sufficient length and
 *  returned storing the qualified name of instance inst
 *  with respect to the ancestor instance ref which utilizes
 *  the fewest intermediate instance names.  If NULL is
 *  passed in for ref, the full qualified name of the
 *  instance with respect to the root instance of the simulation
 *  is returned.  Otherwise, if the subtree inst..ref is found
 *  to not exist, "?" is returned.  The string should be
 *  destroyed when it is no longer in use.
 *
 *  asc_make_any_instance_name():
 *  The same holds here as above except any one of the alias
 *  names for the inst may be returned and not necessarily
 *  that which utilizes the shortest path between inst and ref.
 *  This will be expected to perform faster.
 */

extern struct Instance *asc_get_instance();
/*
 *  inst = asc_get_instance(name, ref)
 *  struct Instance *inst, *ref;
 *  char *name;
 *
 *  Parses the given qualified name string into its component
 *  parts, returning the instance denoted by the entire path of
 *  intermediate instance names.  The first instance name
 *  in the string is expected to be an existing child of the
 *  instance ref.  If there is a parse error or if the instance
 *  cannot be found, NULL is returned.
 */

extern struct Instance *asc_get_child_by_name();
/*
 *  child = asc_get_child_by_name(inst, name)
 *  struct Instance *child, *inst;
 *  char *name;
 *
 *  Searches the instance for child member with the given name.
 *  If it can't be found, NULL is returned.
 */

extern char *asc_make_dimensions();
/*
 *  dimens = asc_make_dimensions(dim)
 *  char *dimens;
 *  dim_type *dim;
 *
 *  Prints the dimensions to a sufficiently long string which
 *  is created and returned.  The string should be destroyed when
 *  no longer in use.
 */

extern int g_check_dimensions_noisy;
/*
 *  if 0, warnings are suppressed. if 1, warnings are given
 *  from asc_check_dimensions();
 */

extern int asc_check_dimensions();
/*
 *  valid = asc_check_dimensions(rel,dimens);
 *  int valid;
 *  struct relation *rel;
 *  dim_type *dimens;
 *
 *  THIS ONLY WORKS ON e_token relations and later for e_opcode
 *  relations. rel is assumed to be valid when called. !!!
 *
 *  Scans a relation in postfix and collects all dimensional
 *  information by applying each token.  It returns a value of TRUE
 *  only if no real instances or real atom instances with wild
 *  dimensionality and no dimensional inconsistencies were encountered.
 *  If the return value is 2 rather than 1, then the dimensionality
 *  has been determined before.
 *  The address of an allocated dimension type is passed in so that
 *  the dimensions of the relation (or at least what the function
 *  thinks the dimensions ought to be) can be also obtained.
 */
#endif /* _OLD_UTILS_H */
