# parameterized collocation test script
# by  Ken Tyner, May 15, 1997.
# This reads in previously saved columns and converges them into
# a flowsheet with recycle.

#The following procedure is used for writting documentation
# to the command line
proc doc {line} {
puts "#############################################"
puts $line
puts "#############################################"
}

doc "
Setting non-default solver parameters
"
set ascSolvQRSlvVect(scaleopt) RELNOM
set ascSolv32767Vect(update_time) 50
set ascSolv32767Vect(update_frequency) 1000

doc "
Setting path variables and reading files into library
"

set lib $env(ASCENDDIST)/models/libraries
set ex $env(ASCENDDIST)/models/examples
set mod $env(ASCENDDIST)/models

DELETE TYPES;
set startc [exec date]
READ FILE "$lib/system.lib";
READ FILE "$lib/atoms.lib";
READ FILE "$lib/components.par.lib";
READ FILE "$lib/propertyoptions.par.lib";
READ FILE "$lib/HGthermo.par.lib";
READ FILE "$lib/stream.par.lib";
READ FILE "$lib/flash.par.lib";
READ FILE "$mod/cost_column.a4l";
READ FILE "$lib/plot.par.lib"
READ FILE "$mod/ternary_plot.a4l";
READ FILE "$mod/KenPendings.a4l";
READ FILE "$mod/collocation_plot.a4l";
READ FILE "$mod/abc_flowsheet.a4l";

set start [exec date]
doc "
Compiling flowsheet model and exporting to the Browser
"
COMPILE {tc} OF abc_demo_sheet;
BROWSE {tc};
set end [exec date]

puts "COMPILE TIMES:"
puts "     start time:  $start"
puts "     end time:   $end"

doc "
Reading a previously saved values file containing three columns
solved to equilibrium and heat balance.  Note that although the
individual columns are converged, the flowsheet is not.
"
READ VALUES $mod/abc_sheet.a4v;

doc "
Plotting columns on ternary diagram
"
RUN {tc.eq_plot.values};
PLOT {tc.eq_plot.plot} ;
SHOW LAST;

doc "
The flowsheet we are using can be dynamically reconfigured.
Here we are making connections between the columns and 
fully recycling the azeotrope.
"
RUN {tc.abc_sheet.remove_connections};
#sheet feed to col 1 feed
ASSIGN {tc.abc_sheet.stream_exists[7][1]} TRUE {*};
#col 2 dist to col 3 feed
ASSIGN {tc.abc_sheet.stream_exists[2][3]} TRUE {*};
#col 1 bot to col 2 feed
ASSIGN {tc.abc_sheet.stream_exists[4][2]} TRUE {*};

#col 3 bot to col 1
ASSIGN {tc.abc_sheet.stream_exists[6][1]} TRUE {*};
#col 1 dist to prod
ASSIGN {tc.abc_sheet.stream_exists[1][4]} TRUE {*};
#col 2 bot to prod
ASSIGN {tc.abc_sheet.stream_exists[5][5]} TRUE {*};
#col 3 dist to prod
ASSIGN {tc.abc_sheet.stream_exists[3][6]} TRUE {*};

ASSIGN {tc.abc_sheet.feed_split[1].split[1]} 1 {};
ASSIGN {tc.abc_sheet.dist_split[1].split[4]} 1 {};
ASSIGN {tc.abc_sheet.dist_split[2].split[3]} 1 {};
ASSIGN {tc.abc_sheet.dist_split[3].split[6]} 1 {};
ASSIGN {tc.abc_sheet.bot_split[1].split[2]} 1 {};
ASSIGN {tc.abc_sheet.bot_split[2].split[5]} 1 {};
#next line enforces total recycle
ASSIGN {tc.abc_sheet.bot_split[3].split[1]} 1 {};

ASSIGN {tc.abc_sheet.reduce[1]} 1.0 {};
ASSIGN {tc.abc_sheet.reduce[2]} 1.0 {};
ASSIGN {tc.abc_sheet.reduce[3]} 1.0 {};

doc "
Running procedures to configure splitters and mixers
and set degrees of freedom for the flowsheet.
"
RUN {tc.abc_sheet.reset};
RUN {tc.abc_sheet.set_connections};
RUN {tc.abc_sheet.reduce_mix_diff};
RUN {tc.abc_sheet.scale};

doc "
Converging the flowsheet.
"
set start [exec date]

SOLVE {tc} WITH QRSlv;

set end [exec date]

puts "SOLVE TIMES:"
puts "     start time:  $start"
puts "     end time:   $end"

doc "
Plotting columns on ternary diagram
"
PLOT {tc.eq_plot.plot} ;
SHOW LAST;

##############################################


doc "
Running scale procedure at converged solution.
"
Solve_do_Flush;
RUN {tc.abc_sheet.scale};

doc "
Changing design variables:
Column 1:
	Distillate Flow	3.5 => 3.45
	RR		4 => 8
	Rect Trays 	20 => 25
	Strip Trays 	15 => 20
Column 3:
	RR		4 => 5
	Rect Trays	10 => 20
"
ASSIGN {tc.abc_sheet.Distillate[1].Ftot} 3.45 {mole/second};
ASSIGN {tc.abc_sheet.Column[1].condenser.reflux_ratio} 8 {};
ASSIGN {tc.abc_sheet.Column[1].s_stack[1]} 25 {};
ASSIGN {tc.abc_sheet.Column[1].s_stack[2]} 20 {};

ASSIGN {tc.abc_sheet.Column[3].condenser.reflux_ratio} 5 {};
ASSIGN {tc.abc_sheet.Column[3].s_stack[1]} 20 {};

doc "
Solving flowsheet with new design variable values.
"
set start [exec date]

SOLVE {tc} WITH QRSlv;

set end [exec date]
puts "SOLVE TIMES:"
puts "     start time:  $start"
puts "     end time:   $end"

doc "
Plotting columns on ternary diagram
"
PLOT {tc.eq_plot.plot} ;
SHOW LAST;











